@@ -27,7 +27,7 @@ class Config(object):
 	recv_chunk = 4096
 	list_md5 = False
 	human_readable_sizes = False
-	extra_headers = SortedDict()
+	extra_headers = SortedDict(ignore_case = True)
 	force = False
 	get_continue = False
 	skip_existing = False
@@ -68,6 +68,7 @@ class Config(object):
 	debug_exclude = {}
 	debug_include = {}
 	encoding = "utf-8"
+	urlencoding_mode = "normal"
 
 	## Creating a singleton
 	def __new__(self, configfile = None):
